package com.diss.omppapp.services;

import com.diss.omppapp.entities.ClassEntity;
import com.diss.omppapp.entities.LecturerClassessEntity;
import com.diss.omppapp.entities.StudentClassesEntity;
import com.diss.omppapp.repositories.ClassEntityRepository;
import com.diss.omppapp.repositories.LecturerClassesEntityRepository;
import com.diss.omppapp.repositories.StudentClassesEntityRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

@Component
public class ClassInfoService {

    @Autowired
    private ClassEntityRepository classEntityRepository;

    @Autowired
    private LecturerClassesEntityRepository lecturerClassesEntityRepository;

    @Autowired
    private StudentClassesEntityRepository studentClassesEntityRepository;

    public List<ClassEntity> getClasses(String username) {
        if(username == null || username.isEmpty()) {
            return new ArrayList<>();
        }

        List<ClassEntity> response = new ArrayList<>();

        if(isStudent(username) && !isLecturer(username)) {
            getStudentClasses(username).forEach(e -> {
                response.add(classEntityRepository.findByClassCode(e.getClassCode()).get());
            });
        } else if(!isStudent(username) && isLecturer(username)) {
            getLecturerClasses(username).forEach( e -> {
                response.add(classEntityRepository.findByClassCode(e.getClassCode()).get());
            });
        } else {
            return  response;
        }

        return response;
    }

    private Boolean isStudent(String username) {
        return studentClassesEntityRepository.existsByUsername(username);
    }

    private Boolean isLecturer(String username) {
        return lecturerClassesEntityRepository.existsByUsername(username);
    }

    private List<StudentClassesEntity> getStudentClasses(String username) {
        Optional<List<StudentClassesEntity>> classes = studentClassesEntityRepository.findAllByUsername(username);
        return classes.orElseGet(ArrayList::new);
    }

    private List<LecturerClassessEntity> getLecturerClasses(String username) {
        Optional<List<LecturerClassessEntity>> classes = lecturerClassesEntityRepository.findAllByUsername(username);
        return classes.orElseGet(ArrayList::new);
    }
}
