package com.diss.omppapp.services;

import com.diss.omppapp.entities.MinutePaperEntity;
import com.diss.omppapp.entities.QuestionEntity;
import com.diss.omppapp.pojo.PaperDetails;
import com.diss.omppapp.pojo.PaperDetailsWrapper;
import com.diss.omppapp.pojo.QuestionDetails;
import com.diss.omppapp.repositories.AnswerEntityRepository;
import com.diss.omppapp.repositories.MinutePaperEntityRepository;
import com.diss.omppapp.repositories.QuestionEntityRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import javax.swing.text.html.Option;
import javax.transaction.Transactional;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Optional;

@Component
public class LecturerInfoService {

    @Autowired
    private MinutePaperEntityRepository minutePaperEntityRepository;

    @Autowired
    private QuestionEntityRepository questionEntityRepository;

    @Autowired
    private AnswerEntityRepository answerEntityRepository;

    public List<MinutePaperEntity> getClassPapers(String username, String classCode) {
        if((username == null || username.isEmpty()) || (classCode == null || classCode.isEmpty())) {
            return new ArrayList<>();
        }

        Optional<List<MinutePaperEntity>> papers = minutePaperEntityRepository.findAllByUsernameAndClassCodeAndTemplate(username,classCode, false);
        return papers.orElseGet(ArrayList::new);
    }

    public List<PaperDetailsWrapper> getTemplates(String username) {
        if(username == null || username.isEmpty()) {
            return new ArrayList<>();
        }

        List<PaperDetailsWrapper> templates = new ArrayList<>();
        List<PaperDetails> templatePapers = new ArrayList<>();
        Optional<List<MinutePaperEntity>> papers = minutePaperEntityRepository.findAllByUsernameAndTemplate(username, true);

        papers.ifPresent(paper -> {
            paper.forEach(elem -> {
                templatePapers.add(new PaperDetails(String.valueOf(elem.getPaperId()), elem.getUsername(), elem.getClassCode(), elem.getName(), elem.getTopic(),
                        elem.isAnonymous(), elem.isTemplate(),elem.getCreationDate().toString()));
            });
        });

        for(int i = 0; i < templatePapers.size(); i++) {
            Optional<List<QuestionEntity>> questionEntities = questionEntityRepository.findAllByPaperIdOrderByQuestionIdAsc(parseId(templatePapers.get(i).getPaperId()));
            List<QuestionDetails> questionDetails = new ArrayList<>();
            questionEntities.ifPresent(elem -> {
                elem.forEach(q -> {
                    questionDetails.add(new QuestionDetails(String.valueOf(q.getPaperId()),String.valueOf(q.getQuestionId()), q.getQuestionText()));
                });
            });
            templates.add(new PaperDetailsWrapper(templatePapers.get(i), questionDetails));
        }

        return templates;
    }

    public void create(PaperDetailsWrapper wrapper, Boolean isTemplate) {
        try {
            boolean exists = minutePaperEntityRepository.existsByPaperId(parseId(wrapper.getPaper().getPaperId()));
            int id = parseId(wrapper.getPaper().getPaperId());
            id = createPaper(id, wrapper.getPaper(), exists, isTemplate);
            createQuestions(id, wrapper.getQuestions());
        } catch (Exception e) {
            //Tear down
            e.printStackTrace();
        }
    }

    private int createPaper(int id ,PaperDetails paperDetails, boolean exists, boolean isTemplate) throws Exception {
        MinutePaperEntity entity = new MinutePaperEntity();
        if(exists) {
            entity = minutePaperEntityRepository.findByPaperId(id).get();
            entity.setPaperId(entity.getPaperId());
        } else {
            entity.setUsername(paperDetails.getUsername());
            entity.setCreationDate(new Date());
        }
        entity.setClassCode(paperDetails.getClassCode());
        entity.setAnonymous(paperDetails.isAnonymous());
        entity.setName(paperDetails.getName());
        entity.setTopic(paperDetails.getTopic());
        entity.setTemplate(isTemplate);
        return minutePaperEntityRepository.save(entity).getPaperId();
    }

    private void createQuestions(int paperId, List<QuestionDetails> questions) throws Exception {
        Optional<List<QuestionEntity>> questionEntities = questionEntityRepository.findAllByPaperIdOrderByQuestionIdAsc(paperId);

        if(questionEntities.isPresent()) {
            if(questionEntities.get().size() > questions.size()) {
                questionEntityRepository.deleteByPaperId(paperId);
            }

            for (int j = 0; j < questions.size(); j++) {
                QuestionEntity question;
                if(questionEntityRepository.existsByQuestionId(parseId(questions.get(j).getQuestionId()))) {
                    question = questionEntityRepository.findByQuestionId(parseId(questions.get(j).getQuestionId())).get();
                } else {
                    question = new QuestionEntity();
                    question.setPaperId(paperId);
                }
                question.setQuestionText(questions.get(j).getQuestionText());
                questionEntityRepository.save(question);
            }
        } else {
            questions.forEach(question -> {
                QuestionEntity entity = new QuestionEntity();
                entity.setPaperId(paperId);
                entity.setQuestionText(question.getQuestionText());
                questionEntityRepository.save(entity);
            });
        }
    }

    @Transactional
    public void delete(String paperId, Boolean isTemplate) {
        if(paperId != null && !paperId.isEmpty()) {
            int id = parseId(paperId);
            if(!isTemplate) {
                answerEntityRepository.deleteByPaperId(id);
            }
            questionEntityRepository.deleteByPaperId(id);
            minutePaperEntityRepository.deleteByPaperId(id);
        }
    }

    private int parseId(String paperId) {
        int id = 0;

        try {
            id = Integer.parseInt(paperId);
        } catch (NumberFormatException nfe) {
            return -1;
        }
        return id;
    }
}
