import {Component, OnChanges, OnDestroy, OnInit, SimpleChanges} from '@angular/core';
import {LoginService} from '../services/login.service';
import {Router} from '@angular/router';
import {takeWhile} from 'rxjs/operators';

@Component({
  selector: 'app-login',
  templateUrl: './login.component.html',
  styleUrls: ['./login.component.css']
})
export class LoginComponent implements OnInit, OnDestroy, OnChanges {
  public username: String;
  public password: String;
  public isUsernameInvalid: Boolean;
  public isPasswordInvalid: Boolean;
  public isSubmitted: Boolean;
  private alive = true;

  constructor(private loginService: LoginService,
              private router: Router) {}

  ngOnInit() {
    this.username = '';
    this.password = '';
    this.isUsernameInvalid = true;
    this.isPasswordInvalid = true;
    this.isSubmitted = false;
  }


  ngOnChanges(changes: SimpleChanges): void {
    this.validateUsername();
    this.validatePassword();
  }

  ngOnDestroy(): void {
    this.username = '';
    this.password = '';
    this.alive = false;
  }

  public validateUsername() {
    this.isSubmitted = false;
    if (this.username === undefined || this.username.length === 0 || this.username === '') {
      this.isUsernameInvalid = true;
    } else {
      this.isUsernameInvalid = false;
    }
  }

  public validatePassword() {
    this.isSubmitted = false;
    if (this.password === undefined || this.password.length === 0 || this.password === '') {
      this.isPasswordInvalid = true;
    } else {
      this.isPasswordInvalid = false;
    }
  }

  private login(): void {
    this.isSubmitted = true;
    if (this.isUsernameInvalid || this.isPasswordInvalid) {
      return;
    }
    this.loginService.login(this.username, this.password)
      .pipe(
        takeWhile(() => this.alive)
      )
      .subscribe((data) => {
        if (data['auth'] === 'true') {
            if (data['lec'] === 'true') {
              this.router.navigate(['/ldashboard']);
              sessionStorage.setItem('lecturer', 'true');
            } else {
              this.router.navigate(['/sdashboard']);
              sessionStorage.setItem('lecturer', 'false');
            }
            sessionStorage.setItem('username', this.username.toString());
        }
      });
  }
}
